import hmac
import hashlib
from datetime import datetime
import math
import hashlib
from Crypto.Cipher import AES
import base64

class Security:
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls)

    def genarateSign(self, data, clientId, timestamp, secretKey):
        message = clientId + "|" + str(timestamp) + "|" + str(data)
        signature = hmac.new(
            bytes(secretKey, 'utf-8'),
            msg=bytes(message, 'utf-8'),
            digestmod=hashlib.sha256
        ).hexdigest()
        return signature

    def verifySign(self, data: str, dataVilidate: str, clientId: str, timestamp: int, secretKey: str) -> bool:
        dt = datetime.now()
        if (timestamp > math.floor(datetime.timestamp(dt) * 1000)):
            return False
        sign = self.genarateSign(data, clientId, timestamp, secretKey)
        if (dataVilidate != sign):
            return False
        return True

    def aesEcrypt(self, data: str, encryptKey: str):
        key = bytes.fromhex(encryptKey)
        data_bytes = bytes(data, 'utf8')
        data_pad = self.__pad(data_bytes)
        cipher = AES.new(key, AES.MODE_CBC, key[0:16])
        encrypt = cipher.encrypt(data_pad)
        return base64.b64encode(encrypt)

    def aesDecrypt(self, data: str, encryptKey: str):
        key = bytes.fromhex(encryptKey)
        message = base64.b64decode(data)
        cipher = AES.new(key, AES.MODE_CBC, key[0:16])
        decrypt = cipher.decrypt(message)
        return self.__unpad(decrypt.decode('utf-8'))

    def __pad(self,byte_array:bytearray, block_size: int=16):
        """
        pkcs5 padding
        """
        pad_len = block_size - len(byte_array) % block_size
        return byte_array + (bytes([pad_len]) * pad_len)

    def __unpad(self,byte_array:bytearray):
        return byte_array[:-ord(byte_array[-1:])]


class CustomerInfo: 
    fullName: str
    email: str
    phone: str
    address: str

    def __init__(self, fullName, email, phone, address):
        self.fullName = fullName
        self.email = email
        self.phone = phone
        self.address = address
        
    def __str__(self) -> str:
        return {
            "fullName": self.fullName,
            "email": self.email,
            "phone": self.phone,
            "address": self.address
        }


import json
import math
from datetime import datetime
from typing import TypeVar

import requests

from src.exception.CustomException import CustomException
from src.model.CancelPayment import *
from src.model.CheckPayment import *
from src.model.CreatePayment import *
from src.service.security import Security

T = TypeVar('T')

class Payment:
    __baseUrl = "https://api-umeecore-dev.hcm.unicloud.ai/umee-pay"
    __endpointCreate = "/api/payment/v1/create"
    __endpointCheck = "/api/payment/v1/check"
    __endpointCancel = "/api/payment/v1/cancel"
    __clientId: str
    __secretKey: str
    __encryptKey: str
    __security: Security = Security()

    def __init__(self, clientId: str, encryptKey: str, secretKey: str):
        self.__clientId = clientId
        self.__secretKey = secretKey
        self.__encryptKey = encryptKey

    def __excute(self, url: str, data: T) -> str:
        dt = datetime.now()
        timestamp = math.floor(datetime.timestamp(dt) * 1000)
        print(data.__str__())
        dataJson = json.dumps(data.__str__())
        payload = self.__security.aesEcrypt(dataJson, self.__encryptKey).decode('UTF-8')
        apiValidate = self.__security.genarateSign(payload, self.__clientId, timestamp, self.__secretKey)
        requestData = {
            "data": payload
        }
        response = requests.post(url,  json.dumps(requestData), headers={
            'x-api-client': self.__clientId,
            'x-api-validate': apiValidate,
            'x-api-time': str(timestamp),
            'Content-Type': 'application/json',
        })
        responseData = response.content.decode('UTF-8')

        clientRes = str(response.headers['x-api-client'])
        dataValidateRes = str(response.headers['x-api-validate'])
        timestampRes = int(response.headers['x-api-time'])
        responseData = json.loads(responseData)

        dataDescrypt = responseData['data']

        if self.__clientId != clientRes:
            raise CustomException(1, "Invalid Client")

        if responseData['data'] == None:
            raise CustomException(responseData['code'], responseData['message'])

        if self.__security.verifySign(responseData['data'], dataValidateRes, clientRes, timestampRes, self.__secretKey) == False:
            raise CustomException(2, "Invalid Security")

        dateDecrypt = self.__security.aesDecrypt(dataDescrypt, self.__encryptKey)
        return dateDecrypt


    def create_payment(self, data: CreatePaymentRequest) -> CreatePaymentResponse:
        url = self.__baseUrl + self.__endpointCreate
        response: str = self.__excute(url,data)
        return CreatePaymentResponse(response)

    def check_payment(self, data: CheckPaymentRequest) -> CheckPaymentResponse:
        url = self.__baseUrl + self.__endpointCheck
        resposne: str = self.__excute(url, data)
        return CheckPaymentResponse(resposne)

    def cancel_payment(self, data: CancelPaymentRequest) -> CancelPaymentResponse:
        url = self.__baseUrl + self.__endpointCancel
        resposne: str = self.__excute(url, data)
        return CancelPaymentResponse(resposne)



import json

class CancelPaymentRequest:
    transactionId: str

    def __init__(self, transactionId: str) -> None:
        self.transactionId = transactionId

    def __str__(self) -> str:
        return {
            "transactionId": self.transactionId
        }

class CancelPaymentResponse: 
    success: bool     

    def __str__(self) -> str:
        return {
            "success": self.success
        }
    
    def __init__(self, j) -> None:
        self.__dict__ = json.loads(j) 


from src.model.CustomerInfo import *
import json

class CreatePaymentRequest :
    refTransactionId: str
    amount: int
    description: str
    timeout: int
    title: str
    language: str
    customerInfo: CustomerInfo
    successUrl: str
    failUrl: str
    redirectAfter: int
    bankAccountId: str

    def __init__(self, refTransactionId: str, amount: int, description: str, timeout: str, title: str, customerInfo: CustomerInfo,
    successUrl: str, failUrl: str, redirectAfter: str, language: str = None, bankAccountId: str = None) -> None:
        self.refTransactionId = refTransactionId
        self.amount = amount
        self.description = description
        self.timeout = timeout
        self.title = title
        self.customerInfo = customerInfo
        self.successUrl = successUrl
        self.failUrl = failUrl
        self.redirectAfter = redirectAfter
        self.language = language
        self.bankAccountId = bankAccountId

    def __str__(self) -> str:
        return {
            "refTransactionId": self.refTransactionId,
            "amount": self.amount,
            "description": self.description,
            "timeout": self.timeout,
            "title": self.title,
            "language": self.language,
            "customerInfo": self.customerInfo.__str__(),
            "successUrl": self.successUrl,
            "failUrl": self.failUrl,
            "redirectAfter": self.redirectAfter,
        }


class CreatePaymentResponse: 
    transactionId: str
    refTransactionId: str
    payLinkCode: str
    timeout: int
    url: str
    virtualAccount: str
    description: str
    amount: int
    qrCodeString: str
    status: str
    time: str

    def __init__(self, transactionId, refTransactionId, payLinkCode, timeout, url, virtualAccount, description, amount, qrCodeString, status, time) -> None:
        self.transactionId = transactionId
        self.refTransactionId = refTransactionId
        self.payLinkCode = payLinkCode
        self.timeout = timeout
        self.url = url
        self.virtualAccount = virtualAccount
        self.description = description
        self.amount = amount
        self.qrCodeString = qrCodeString
        self.status = status
        self.time = time

    def __init__(self, j) -> None:
        self.__dict__ = json.loads(j)


import json

class CheckPaymentRequest: 
    transactionId: str

    def __init__(self, transactionId: str) -> None:
        self.transactionId = transactionId

    def __str__(self) -> str:
        return {
            "transactionId": self.transactionId
        }

class CheckPaymentResponse:
    status: str
    refTransactionId: str
    amount: int

    def __str__(self) -> str:
        return {
            "status": self.status,
            "refTransactionId": self.refTransactionId,
            "amount": self.amount
        }

    def __init__(self, j) -> None:
        self.__dict__ = json.loads(j) 


class CustomException(Exception) :
    __code: int
    __message: str

    def __init__(self, code:int, message: str):
        super().__init__(message)
        self.__message = message
        self.__code = code

    def get_code(self) -> int:
        return self.__code

    def get_message(self) -> str:
        return self.__message